using System;
using System.Windows.Forms;
using Microsoft.Win32.SafeHandles;

namespace WinUsbDemo
{
	/// <summary>
	/// Project: WinUsb_cs
	/// 
	/// ***********************************************************************
	/// Software License Agreement
	///
	/// Licensor grants any person obtaining a copy of this software ("You") 
	/// a worldwide, royalty-free, non-exclusive license, for the duration of 
	/// the copyright, free of charge, to store and execute the Software in a 
	/// computer system and to incorporate the Software or any portion of it 
	/// in computer programs You write.   
	/// 
	/// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
	/// IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
	/// FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
	/// AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
	/// LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
	/// OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
	/// THE SOFTWARE.
	/// ***********************************************************************
	/// 
	/// Author
	/// Jan Axelson          
	/// 
	/// This software was created using Visual Studio 2012 Professional Edition with .NET Framework 4.0.
	/// 
	/// Purpose: 
	/// Demonstrates USB communications using Microsoft's WinUSB driver.
	/// 
	/// Requirements:
	/// Windows XP or later and an attached USB device that uses the WinUSB driver.
	/// 
	/// Description:
	/// Finds an attached device whose device firmware or host INF file contains a specific device interface GUID.
	/// Enables sending and receiving data via bulk, interrupt, and control transfers.
	/// 
	/// Uses RegisterDeviceNotification and WM_DEVICE_CHANGE messages
	/// to detect when a device is attached or removed.
	/// 
	/// For bulk transfers, the application uses a Delegate and the BeginInvoke 
	/// and EndInvoke methods to read and write data asynchronously so the application's main thread 
	/// doesn't have to wait for the device to return data. A callback routine uses 
	/// marshaling to send data to the form, whose code runs in a different thread.
	/// 
	/// Interrupt and control transfers read and write data synchronously, blocking the thread
	/// until the operation completes or a timeout. (Normally, the wait for completion is minimal.)
	///  
	/// This software, an example INF file, and companion device firmware are available from 
	/// www.Lvr.com
	/// 
	/// Send bug reports to jan@Lvr.com  
	/// This application has been tested under Windows XP (32 bit) and Windows 8 (64 bit).
	/// 
	/// 5/3/13
	///	V2.0
	///	To illustrate asynchronous operations, bulk read and write are asynchronous.
	///	
	///	To illustrate synchronous operations, interrupt read and write and control transfers are synchronous. 
	///	
	/// For compatibility, replaced ToInt32 with ToInt64 here:
	/// IntPtr pDevicePathName = new IntPtr(detailDataBuffer.ToInt64() + 4);
	/// and here:
	/// if ((deviceNotificationHandle.ToInt64() == IntPtr.Zero.ToInt64()))
	/// 
	/// Uncommented QueryDeviceSpeed because Microsoft corrected the documentation for it.
	/// 
	/// Renamed some routines and variables.
	/// 
	/// Moved myDevInfo and other variables from WinUsbDevice.cs to frmMain.cs. 
	/// 
	/// For compatibility if the charset isn't English, added System.Globalization.CultureInfo.InvariantCulture here:
	/// if ((String.Compare(DeviceNameString, mydevicePathName, true, System.Globalization.CultureInfo.InvariantCulture) == 0))
	/// 
	/// Moved API declarations to classes named NativeMethods as recommended by Microsoft.
	/// 
	/// Minor changes to API declarations as advised by Visual Studio's Code Analysis.
	/// 
	/// Naming changes and other minor coding improvements as recommended by the Resharper tool.
	/// </summary>

	/// <summary>
	/// The application's form. Buttons enable detecting a device
	/// and initiating transfers.
	/// </summary>
	/// 
	internal class FrmMain : Form
	{
		#region '"Windows Form Designer generated code "'
		public FrmMain()
			: base()
		{


			// This call is required by the Windows Form Designer.
			InitializeComponent();
		}
		// Form overrides dispose to clean up the component list.
		protected override void Dispose(Boolean Disposing)
		{
			if (Disposing)
			{
				if (components != null)
				{
					components.Dispose();
				}
			}
			base.Dispose(Disposing);
		}

		// Required by the Windows Form Designer
		private System.ComponentModel.IContainer components;
		public System.Windows.Forms.ToolTip ToolTip1;
		public System.Windows.Forms.Button CmdSendandReceiveViaBulkTransfers;
		public System.Windows.Forms.GroupBox FraControlReadTransfer;
		public System.Windows.Forms.TextBox TxtBulkDataToSend;
		public System.Windows.Forms.GroupBox FraBulkTransfer;
		public System.Windows.Forms.ComboBox CboInterruptOutByte1;
		public System.Windows.Forms.ComboBox CboInterruptOutByte0;
		public System.Windows.Forms.GroupBox FraInterruptTransfer;
		public System.Windows.Forms.ListBox LstResults;
		// NOTE: The following procedure is required by the Windows Form Designer
		// It can be modified using the Windows Form Designer.
		// Do not modify it using the code editor.
		internal System.Windows.Forms.Button CmdFindDevice;
		internal System.Windows.Forms.Button CmdSendAndReceiveViaInterruptTransfers;
		public System.Windows.Forms.GroupBox FraControWriteTransfer;
		internal System.Windows.Forms.Button CmdControlWriteTransfer;
		public System.Windows.Forms.ComboBox CboControlWriteByte1;
		public System.Windows.Forms.ComboBox CboControlWriteByte0;
		internal System.Windows.Forms.Button CmdControlReadTransfer;

		[System.Diagnostics.DebuggerStepThrough()]
		private void InitializeComponent()
		{
			this.components = new System.ComponentModel.Container();
			this.ToolTip1 = new System.Windows.Forms.ToolTip(this.components);
			this.FraControlReadTransfer = new System.Windows.Forms.GroupBox();
			this.CmdControlReadTransfer = new System.Windows.Forms.Button();
			this.CmdSendandReceiveViaBulkTransfers = new System.Windows.Forms.Button();
			this.FraBulkTransfer = new System.Windows.Forms.GroupBox();
			this.TxtBulkDataToSend = new System.Windows.Forms.TextBox();
			this.FraInterruptTransfer = new System.Windows.Forms.GroupBox();
			this.CmdSendAndReceiveViaInterruptTransfers = new System.Windows.Forms.Button();
			this.CboInterruptOutByte1 = new System.Windows.Forms.ComboBox();
			this.CboInterruptOutByte0 = new System.Windows.Forms.ComboBox();
			this.LstResults = new System.Windows.Forms.ListBox();
			this.CmdFindDevice = new System.Windows.Forms.Button();
			this.FraControWriteTransfer = new System.Windows.Forms.GroupBox();
			this.CboControlWriteByte1 = new System.Windows.Forms.ComboBox();
			this.CboControlWriteByte0 = new System.Windows.Forms.ComboBox();
			this.CmdControlWriteTransfer = new System.Windows.Forms.Button();
			this.FraControlReadTransfer.SuspendLayout();
			this.FraBulkTransfer.SuspendLayout();
			this.FraInterruptTransfer.SuspendLayout();
			this.FraControWriteTransfer.SuspendLayout();
			this.SuspendLayout();
			// 
			// fraControlReadTransfer
			// 
			this.FraControlReadTransfer.BackColor = System.Drawing.SystemColors.Control;
			this.FraControlReadTransfer.Controls.Add(this.CmdControlReadTransfer);
			this.FraControlReadTransfer.Font = new System.Drawing.Font("Arial", 9.75F, System.Drawing.FontStyle.Regular, System.Drawing.GraphicsUnit.Point, ((byte)(0)));
			this.FraControlReadTransfer.ForeColor = System.Drawing.SystemColors.ControlText;
			this.FraControlReadTransfer.Location = new System.Drawing.Point(237, 27);
			this.FraControlReadTransfer.Name = "FraControlReadTransfer";
			this.FraControlReadTransfer.RightToLeft = System.Windows.Forms.RightToLeft.No;
			this.FraControlReadTransfer.Size = new System.Drawing.Size(175, 94);
			this.FraControlReadTransfer.TabIndex = 7;
			this.FraControlReadTransfer.TabStop = false;
			this.FraControlReadTransfer.Text = "Control Read Transfer";
			// 
			// cmdControlReadTransfer
			// 
			this.CmdControlReadTransfer.Font = new System.Drawing.Font("Arial", 9.75F, System.Drawing.FontStyle.Regular, System.Drawing.GraphicsUnit.Point, ((byte)(0)));
			this.CmdControlReadTransfer.Location = new System.Drawing.Point(49, 28);
			this.CmdControlReadTransfer.Name = "CmdControlReadTransfer";
			this.CmdControlReadTransfer.Size = new System.Drawing.Size(71, 36);
			this.CmdControlReadTransfer.TabIndex = 12;
			this.CmdControlReadTransfer.Text = "Send";
			this.CmdControlReadTransfer.UseVisualStyleBackColor = true;
			this.CmdControlReadTransfer.Click += new System.EventHandler(this.cmdControlReadTransfer_Click);
			// 
			// cmdSendandReceiveViaBulkTransfers
			// 
			this.CmdSendandReceiveViaBulkTransfers.BackColor = System.Drawing.SystemColors.Control;
			this.CmdSendandReceiveViaBulkTransfers.Cursor = System.Windows.Forms.Cursors.Default;
			this.CmdSendandReceiveViaBulkTransfers.Font = new System.Drawing.Font("Arial", 9.75F, System.Drawing.FontStyle.Regular, System.Drawing.GraphicsUnit.Point, ((byte)(0)));
			this.CmdSendandReceiveViaBulkTransfers.ForeColor = System.Drawing.SystemColors.ControlText;
			this.CmdSendandReceiveViaBulkTransfers.Location = new System.Drawing.Point(256, 24);
			this.CmdSendandReceiveViaBulkTransfers.Name = "CmdSendandReceiveViaBulkTransfers";
			this.CmdSendandReceiveViaBulkTransfers.RightToLeft = System.Windows.Forms.RightToLeft.No;
			this.CmdSendandReceiveViaBulkTransfers.Size = new System.Drawing.Size(71, 36);
			this.CmdSendandReceiveViaBulkTransfers.TabIndex = 8;
			this.CmdSendandReceiveViaBulkTransfers.Text = "Send";
			this.CmdSendandReceiveViaBulkTransfers.UseVisualStyleBackColor = false;
			this.CmdSendandReceiveViaBulkTransfers.Click += new System.EventHandler(this.cmdSendAndReceiveViaBulkTransfers_Click);
			// 
			// fraBulkTransfer
			// 
			this.FraBulkTransfer.BackColor = System.Drawing.SystemColors.Control;
			this.FraBulkTransfer.Controls.Add(this.TxtBulkDataToSend);
			this.FraBulkTransfer.Controls.Add(this.CmdSendandReceiveViaBulkTransfers);
			this.FraBulkTransfer.Font = new System.Drawing.Font("Arial", 9.75F, System.Drawing.FontStyle.Regular, System.Drawing.GraphicsUnit.Point, ((byte)(0)));
			this.FraBulkTransfer.ForeColor = System.Drawing.SystemColors.ControlText;
			this.FraBulkTransfer.Location = new System.Drawing.Point(16, 139);
			this.FraBulkTransfer.Name = "FraBulkTransfer";
			this.FraBulkTransfer.RightToLeft = System.Windows.Forms.RightToLeft.No;
			this.FraBulkTransfer.Size = new System.Drawing.Size(341, 89);
			this.FraBulkTransfer.TabIndex = 4;
			this.FraBulkTransfer.TabStop = false;
			this.FraBulkTransfer.Text = "Bulk Transfers";
			// 
			// txtBulkDataToSend
			// 
			this.TxtBulkDataToSend.AcceptsReturn = true;
			this.TxtBulkDataToSend.BackColor = System.Drawing.SystemColors.Window;
			this.TxtBulkDataToSend.Cursor = System.Windows.Forms.Cursors.IBeam;
			this.TxtBulkDataToSend.Font = new System.Drawing.Font("Arial", 9.75F, System.Drawing.FontStyle.Regular, System.Drawing.GraphicsUnit.Point, ((byte)(0)));
			this.TxtBulkDataToSend.ForeColor = System.Drawing.SystemColors.WindowText;
			this.TxtBulkDataToSend.Location = new System.Drawing.Point(16, 19);
			this.TxtBulkDataToSend.MaxLength = 0;
			this.TxtBulkDataToSend.Multiline = true;
			this.TxtBulkDataToSend.Name = "TxtBulkDataToSend";
			this.TxtBulkDataToSend.RightToLeft = System.Windows.Forms.RightToLeft.No;
			this.TxtBulkDataToSend.ScrollBars = System.Windows.Forms.ScrollBars.Both;
			this.TxtBulkDataToSend.Size = new System.Drawing.Size(224, 59);
			this.TxtBulkDataToSend.TabIndex = 5;
			this.TxtBulkDataToSend.Text = "bulk transfer test data";
			// 
			// fraInterruptTransfer
			// 
			this.FraInterruptTransfer.BackColor = System.Drawing.SystemColors.Control;
			this.FraInterruptTransfer.Controls.Add(this.CmdSendAndReceiveViaInterruptTransfers);
			this.FraInterruptTransfer.Controls.Add(this.CboInterruptOutByte1);
			this.FraInterruptTransfer.Controls.Add(this.CboInterruptOutByte0);
			this.FraInterruptTransfer.Font = new System.Drawing.Font("Arial", 9.75F, System.Drawing.FontStyle.Regular, System.Drawing.GraphicsUnit.Point, ((byte)(0)));
			this.FraInterruptTransfer.ForeColor = System.Drawing.SystemColors.ControlText;
			this.FraInterruptTransfer.Location = new System.Drawing.Point(379, 139);
			this.FraInterruptTransfer.Name = "FraInterruptTransfer";
			this.FraInterruptTransfer.RightToLeft = System.Windows.Forms.RightToLeft.No;
			this.FraInterruptTransfer.Size = new System.Drawing.Size(180, 89);
			this.FraInterruptTransfer.TabIndex = 1;
			this.FraInterruptTransfer.TabStop = false;
			this.FraInterruptTransfer.Text = "Interrupt Transfers";
			// 
			// cmdSendAndReceiveViaInterruptTransfers
			// 
			this.CmdSendAndReceiveViaInterruptTransfers.Font = new System.Drawing.Font("Arial", 9.75F, System.Drawing.FontStyle.Regular, System.Drawing.GraphicsUnit.Point, ((byte)(0)));
			this.CmdSendAndReceiveViaInterruptTransfers.Location = new System.Drawing.Point(98, 24);
			this.CmdSendAndReceiveViaInterruptTransfers.Name = "CmdSendAndReceiveViaInterruptTransfers";
			this.CmdSendAndReceiveViaInterruptTransfers.Size = new System.Drawing.Size(71, 36);
			this.CmdSendAndReceiveViaInterruptTransfers.TabIndex = 7;
			this.CmdSendAndReceiveViaInterruptTransfers.Text = "Send";
			this.CmdSendAndReceiveViaInterruptTransfers.UseVisualStyleBackColor = true;
			this.CmdSendAndReceiveViaInterruptTransfers.Click += new System.EventHandler(this.cmdSendAndReceiveViaInterruptTransfers_Click);
			// 
			// cboInterruptOutByte1
			// 
			this.CboInterruptOutByte1.BackColor = System.Drawing.SystemColors.Window;
			this.CboInterruptOutByte1.Cursor = System.Windows.Forms.Cursors.Default;
			this.CboInterruptOutByte1.DropDownStyle = System.Windows.Forms.ComboBoxStyle.DropDownList;
			this.CboInterruptOutByte1.Font = new System.Drawing.Font("Arial", 8F, System.Drawing.FontStyle.Regular, System.Drawing.GraphicsUnit.Point, ((byte)(0)));
			this.CboInterruptOutByte1.ForeColor = System.Drawing.SystemColors.WindowText;
			this.CboInterruptOutByte1.Location = new System.Drawing.Point(8, 56);
			this.CboInterruptOutByte1.Name = "CboInterruptOutByte1";
			this.CboInterruptOutByte1.RightToLeft = System.Windows.Forms.RightToLeft.No;
			this.CboInterruptOutByte1.Size = new System.Drawing.Size(74, 22);
			this.CboInterruptOutByte1.TabIndex = 3;
			// 
			// cboInterruptOutByte0
			// 
			this.CboInterruptOutByte0.BackColor = System.Drawing.SystemColors.Window;
			this.CboInterruptOutByte0.Cursor = System.Windows.Forms.Cursors.Default;
			this.CboInterruptOutByte0.DropDownStyle = System.Windows.Forms.ComboBoxStyle.DropDownList;
			this.CboInterruptOutByte0.Font = new System.Drawing.Font("Arial", 8F, System.Drawing.FontStyle.Regular, System.Drawing.GraphicsUnit.Point, ((byte)(0)));
			this.CboInterruptOutByte0.ForeColor = System.Drawing.SystemColors.WindowText;
			this.CboInterruptOutByte0.Location = new System.Drawing.Point(8, 24);
			this.CboInterruptOutByte0.Name = "CboInterruptOutByte0";
			this.CboInterruptOutByte0.RightToLeft = System.Windows.Forms.RightToLeft.No;
			this.CboInterruptOutByte0.Size = new System.Drawing.Size(72, 22);
			this.CboInterruptOutByte0.TabIndex = 2;
			// 
			// lstResults
			// 
			this.LstResults.BackColor = System.Drawing.SystemColors.Window;
			this.LstResults.Cursor = System.Windows.Forms.Cursors.Default;
			this.LstResults.Font = new System.Drawing.Font("Arial", 9.75F, System.Drawing.FontStyle.Regular, System.Drawing.GraphicsUnit.Point, ((byte)(0)));
			this.LstResults.ForeColor = System.Drawing.SystemColors.WindowText;
			this.LstResults.HorizontalScrollbar = true;
			this.LstResults.ItemHeight = 16;
			this.LstResults.Location = new System.Drawing.Point(16, 248);
			this.LstResults.Name = "LstResults";
			this.LstResults.RightToLeft = System.Windows.Forms.RightToLeft.No;
			this.LstResults.Size = new System.Drawing.Size(543, 196);
			this.LstResults.TabIndex = 0;
			// 
			// cmdFindDevice
			// 
			this.CmdFindDevice.Font = new System.Drawing.Font("Arial", 9.75F, System.Drawing.FontStyle.Regular, System.Drawing.GraphicsUnit.Point, ((byte)(0)));
			this.CmdFindDevice.Location = new System.Drawing.Point(446, 44);
			this.CmdFindDevice.Name = "CmdFindDevice";
			this.CmdFindDevice.Size = new System.Drawing.Size(102, 59);
			this.CmdFindDevice.TabIndex = 11;
			this.CmdFindDevice.Text = "Find My Device";
			this.CmdFindDevice.Click += new System.EventHandler(this.cmdFindDevice_Click);
			// 
			// fraControWriteTransfer
			// 
			this.FraControWriteTransfer.BackColor = System.Drawing.SystemColors.Control;
			this.FraControWriteTransfer.Controls.Add(this.CboControlWriteByte1);
			this.FraControWriteTransfer.Controls.Add(this.CboControlWriteByte0);
			this.FraControWriteTransfer.Controls.Add(this.CmdControlWriteTransfer);
			this.FraControWriteTransfer.Font = new System.Drawing.Font("Arial", 9.75F, System.Drawing.FontStyle.Regular, System.Drawing.GraphicsUnit.Point, ((byte)(0)));
			this.FraControWriteTransfer.ForeColor = System.Drawing.SystemColors.ControlText;
			this.FraControWriteTransfer.Location = new System.Drawing.Point(16, 25);
			this.FraControWriteTransfer.Name = "FraControWriteTransfer";
			this.FraControWriteTransfer.RightToLeft = System.Windows.Forms.RightToLeft.No;
			this.FraControWriteTransfer.Size = new System.Drawing.Size(193, 96);
			this.FraControWriteTransfer.TabIndex = 12;
			this.FraControWriteTransfer.TabStop = false;
			this.FraControWriteTransfer.Text = "Control Write Transfer";
			// 
			// cboControlWriteByte1
			// 
			this.CboControlWriteByte1.BackColor = System.Drawing.SystemColors.Window;
			this.CboControlWriteByte1.Cursor = System.Windows.Forms.Cursors.Default;
			this.CboControlWriteByte1.DropDownStyle = System.Windows.Forms.ComboBoxStyle.DropDownList;
			this.CboControlWriteByte1.Font = new System.Drawing.Font("Arial", 8F, System.Drawing.FontStyle.Regular, System.Drawing.GraphicsUnit.Point, ((byte)(0)));
			this.CboControlWriteByte1.ForeColor = System.Drawing.SystemColors.WindowText;
			this.CboControlWriteByte1.Location = new System.Drawing.Point(16, 58);
			this.CboControlWriteByte1.Name = "CboControlWriteByte1";
			this.CboControlWriteByte1.RightToLeft = System.Windows.Forms.RightToLeft.No;
			this.CboControlWriteByte1.Size = new System.Drawing.Size(67, 22);
			this.CboControlWriteByte1.TabIndex = 15;
			// 
			// cboControlWriteByte0
			// 
			this.CboControlWriteByte0.BackColor = System.Drawing.SystemColors.Window;
			this.CboControlWriteByte0.Cursor = System.Windows.Forms.Cursors.Default;
			this.CboControlWriteByte0.DropDownStyle = System.Windows.Forms.ComboBoxStyle.DropDownList;
			this.CboControlWriteByte0.Font = new System.Drawing.Font("Arial", 8F, System.Drawing.FontStyle.Regular, System.Drawing.GraphicsUnit.Point, ((byte)(0)));
			this.CboControlWriteByte0.ForeColor = System.Drawing.SystemColors.WindowText;
			this.CboControlWriteByte0.Location = new System.Drawing.Point(16, 30);
			this.CboControlWriteByte0.Name = "CboControlWriteByte0";
			this.CboControlWriteByte0.RightToLeft = System.Windows.Forms.RightToLeft.No;
			this.CboControlWriteByte0.Size = new System.Drawing.Size(67, 22);
			this.CboControlWriteByte0.TabIndex = 14;
			// 
			// cmdControlWriteTransfer
			// 
			this.CmdControlWriteTransfer.Font = new System.Drawing.Font("Arial", 9.75F, System.Drawing.FontStyle.Regular, System.Drawing.GraphicsUnit.Point, ((byte)(0)));
			this.CmdControlWriteTransfer.Location = new System.Drawing.Point(102, 30);
			this.CmdControlWriteTransfer.Name = "CmdControlWriteTransfer";
			this.CmdControlWriteTransfer.Size = new System.Drawing.Size(71, 36);
			this.CmdControlWriteTransfer.TabIndex = 13;
			this.CmdControlWriteTransfer.Text = "Send";
			this.CmdControlWriteTransfer.UseVisualStyleBackColor = true;
			this.CmdControlWriteTransfer.Click += new System.EventHandler(this.cmdControlWriteTransfer_Click);
			// 
			// frmMain
			// 
			this.ClientSize = new System.Drawing.Size(579, 469);
			this.Controls.Add(this.FraControWriteTransfer);
			this.Controls.Add(this.CmdFindDevice);
			this.Controls.Add(this.FraControlReadTransfer);
			this.Controls.Add(this.FraBulkTransfer);
			this.Controls.Add(this.FraInterruptTransfer);
			this.Controls.Add(this.LstResults);
			this.Font = new System.Drawing.Font("Arial", 8F, System.Drawing.FontStyle.Regular, System.Drawing.GraphicsUnit.Point, ((byte)(0)));
			this.Location = new System.Drawing.Point(21, 28);
			this.Name = "FrmMain";
			this.StartPosition = System.Windows.Forms.FormStartPosition.Manual;
			this.Text = "WinUSB Demo";
			this.Load += new System.EventHandler(this.frmMain_Load);
			this.Closed += new System.EventHandler(this.frmMain_Closed);
			this.FraControlReadTransfer.ResumeLayout(false);
			this.FraBulkTransfer.ResumeLayout(false);
			this.FraBulkTransfer.PerformLayout();
			this.FraInterruptTransfer.ResumeLayout(false);
			this.FraControWriteTransfer.ResumeLayout(false);
			this.ResumeLayout(false);

		}

		#endregion

		//  This GUID must match the GUID in the device firmware or host INF file.
		//  To create a GUID in Visual Studio, click Tools > Create GUID.

		// The GUID used in previous versions of this application:
		//private const String WINUSB_DEMO_GUID_STRING = "{42CA71EC-CE1C-44c2-82DE-87D8D8FF6C1E}";

		// New GUID for this version. (Use either one with a matching GUID in an INF file or device firmware.)

		private const String WinusbDemoGuidString = "{ecceff35-146c-4ff3-acd9-8f992d09acdd}";

		private SafeFileHandle _deviceHandle; 
		private IntPtr _deviceNotificationHandle;
		private Boolean _deviceReady;
		private WinUsbCommunications.DeviceInfo _myDeviceInfo = new WinUsbCommunications.DeviceInfo();
		private readonly DeviceManagement _myDeviceManagement = new DeviceManagement();
		private String _myDevicePathName;
		private readonly WinUsbCommunications _myWinUsbCommunications = new WinUsbCommunications();
		private IntPtr _winUsbHandle;

		internal FrmMain FrmMy;
		
		///  <summary>
		///  Define a delegate with the same parameters as AccessForm.
		///  Used in accessing the application's form from a different thread.
		///  </summary>

		private delegate void MarshalToForm(String action, String textToAdd);

		///  <summary>
		///  Define a class of delegates with the same parameters as 
		///  WinUsbDevice.ReadViaBulkTransfer.
		///  Used for asynchronous reads from the device.
		///  </summary>
		
		private delegate void ReceiveFromDeviceDelegate

			(IntPtr winUsbHandle, WinUsbCommunications.DeviceInfo myDeviceInfo, UInt32 bytesToRead, ref Byte[] dataBuffer, ref UInt32 bytesRead, ref Boolean success);

		///  <summary>
		///  Define a class of delegates with the same parameters as 
		///  WinUsbDevice.SendViaBulkTransfer.
		///  Used for asynchronous writes to the device.
		///  </summary>		

		private delegate void SendToDeviceDelegate
			(IntPtr winUsbHandle,
			WinUsbCommunications.DeviceInfo myDevInfo,			
			UInt32 bufferLength,
			Byte[] buffer,
			ref UInt32 lengthTransferred,
			ref Boolean success);
	
		///  <summary>
		///  Performs various application-specific functions that
		///  involve accessing the application's form.
		///  </summary>
		///  
		///  <param name="action"> A String that names the action to perform on the form. </param>
		///  <param name="formText"> Text to display on the form or an empty String. </param>
		///  
		/// <remarks>
		///  In asynchronous calls to WinUsb_ReadPipe, the callback function 
		///  uses this routine to access the application's form, which runs in 
		///  a different thread.
		/// </remarks>
		/// 
		private void AccessForm(String action, String formText)
		{
			//  Select an action to perform on the form:

			switch (action)
			{
				case "AddItemToListBox":

					LstResults.Items.Add(formText);

					break;
				case "AddItemToTextBox":

					TxtBulkDataToSend.SelectedText = formText + "\r\n";

					break;
				case "EnableCmdSendandReceiveViaBulkTransfers":

					CmdSendandReceiveViaBulkTransfers.Enabled = true;
					CmdSendandReceiveViaBulkTransfers.Focus();

					break;
				case "EnableCmdSendandReceiveViaInterruptTransfers":

					CmdSendAndReceiveViaInterruptTransfers.Enabled = true;
					CmdSendAndReceiveViaInterruptTransfers.Focus();

					break;
				case "ScrollToBottomOfListBox":

					LstResults.SelectedIndex = LstResults.Items.Count - 1;

					break;
			}
		}

		///  <summary>
		///  Calls a routine to initiate a Control Read transfer. (Data stage is device to host.)
		///  </summary>

		private void cmdControlReadTransfer_Click(Object sender, EventArgs e)
		{
			RequestControlReadTransfer();
		}

		///  <summary>
		///  Calls a routine to initiate a Control Write transfer. (Data stage is host to device.)
		///  </summary>
		///   
		private void cmdControlWriteTransfer_Click(Object sender, EventArgs e)
		{
			RequestControlWriteTransfer();
		}

		///  <summary>
		///  Search for a specific device.
		///  </summary>

		private void cmdFindDevice_Click(Object sender, EventArgs e)
		{
			FindMyDevice();
		}

		///  <summary>
		///  Calls a routine to exchange data via bulk transfers.
		///  </summary>

		private void cmdSendAndReceiveViaBulkTransfers_Click(Object eventSender, EventArgs eventArgs)
		{
			// Don't allow another transfer request until this one completes.

			CmdSendandReceiveViaBulkTransfers.Enabled = false;

			SendAndReceiveViaBulkTransfers();
		}

		///  <summary>
		///  Calls a routine to exchange data via interrupt transfers.
		///  </summary>
		///  
		private void cmdSendAndReceiveViaInterruptTransfers_Click(Object sender, EventArgs e)
		{
			// Don't allow another transfer request until this one completes.

			CmdSendAndReceiveViaInterruptTransfers.Enabled = false;

			SendAndReceiveViaInterruptTransfers();
		}
		
		///  <summary>
		///  Get and display the device's speed in the list box.
		///  </summary>		

		private void DisplayDeviceSpeed()
		{
			var speed = "";

			WinUsbCommunications.QueryDeviceSpeed(_winUsbHandle, ref _myDeviceInfo);

			switch (_myDeviceInfo.DeviceSpeed)
			{
				case 1:
					speed = "low or full speed";
					break;	
				case 3:
					speed = "high speed or SuperSpeed";
					break;
			}

			LstResults.Items.Add("Device is " + speed);
		}

		///  <summary>
		///  If a device with the specified device interface GUID hasn't been previously detected,
		///  look for it. If found, open a handle to the device.
		///  </summary>
		///  
		///  <returns>
		///  True if the device is detected, False if not detected.
		///  </returns>

		private Boolean FindMyDevice()
		{
			var devicePathName = "";
			Boolean lastDevice;
			const uint pipeTimeout = 2000;

			if (!(_deviceReady))
			{
				// Convert the device interface GUID String to a GUID object: 

				var winUsbDemoGuid =
					new Guid(WinusbDemoGuidString);

				// Fill an array with the device path names of all attached devices with matching GUIDs.

				var deviceFound = _myDeviceManagement.FindDeviceFromGuid
					(winUsbDemoGuid,
					 ref devicePathName);

				if (deviceFound)
				{
					_deviceHandle = _myWinUsbCommunications.GetDeviceHandle(devicePathName);

					if (!_deviceHandle.IsInvalid)
					{
						LstResults.Items.Add("The device has been detected.");

						ScrollToBottomOfListBox();

						_deviceReady = true;

						// Save DevicePathName so OnDeviceChange() knows which name is my device.

						_myDevicePathName = devicePathName;

						// Register to receive notifications if the device is removed or attached.

						var success = _myDeviceManagement.RegisterForDeviceNotifications
							(_myDevicePathName,
							 FrmMy.Handle,
							 winUsbDemoGuid,
							 ref _deviceNotificationHandle);

						if (success)
						{
							_myWinUsbCommunications.InitializeDevice(_deviceHandle, ref _winUsbHandle, ref _myDeviceInfo, pipeTimeout);

							DisplayDeviceSpeed();
						}
					}
					else
					{
						// There was a problem in retrieving the information.

						_deviceReady = false;
						_myWinUsbCommunications.CloseDeviceHandle(_deviceHandle, _winUsbHandle);
						LstResults.Items.Add("Device not found.");
						CmdSendandReceiveViaBulkTransfers.Enabled = true;
						CmdSendAndReceiveViaInterruptTransfers.Enabled = true;
					}
				}
			}
			else
			{
				LstResults.Items.Add("The device has been detected.");
				DisplayDeviceSpeed();
			}

				ScrollToBottomOfListBox();

				return _deviceReady;
		}

		///  <summary>
		///  Perform shutdown operations.
		///  </summary>

		private void frmMain_Closed(Object eventSender, EventArgs eventArgs)
		{
			Shutdown();
		}

		///  <summary>
		///  Perform startup operations.
		///  </summary>

		private void frmMain_Load(Object eventSender, EventArgs eventArgs)
		{
			FrmMy = this;
			Startup();
		}
		
		///  <summary>
		///  Retrieves received data from a bulk endpoint.
		///  This routine is called automatically when myWinUsbDevice.ReadViaBulkTransfer
		///  returns. The routine calls several marshaling routines to access the main form.       
		///  </summary>
		///  
		///  <param name="ar"> An object containing status information about the 
		///  asynchronous operation.</param>
		///  
		private void GetBulkDataSent(IAsyncResult ar)
		{
			UInt32 bytesSent = 0;
			var myEncoder = new System.Text.ASCIIEncoding();
			Byte[] sentDataBuffer;
			String receivedtext = "";
			Boolean success = false;

			// Define a delegate using the IAsyncResult object.

			var deleg =
				((SendToDeviceDelegate)(ar.AsyncState));

			// Get the IAsyncResult object and the values of other paramaters that the
			// BeginInvoke method passed by reference.

			deleg.EndInvoke(ref bytesSent, ref success, ar);

			// Display the received data in the form's list box.

			if ((ar.IsCompleted && success))
			{
				MyMarshalToForm("AddItemToListBox", "Data sent via bulk transfer:");
			}
			else
			{
				MyMarshalToForm("AddItemToListBox", "The attempt to send bulk data has failed.");										
			}

			MyMarshalToForm("ScrollToBottomOfListBox", "");

			// Enable requesting another transfer.

			MyMarshalToForm("EnableCmdSendandReceiveViaBulkTransfers", "");
		}
		
		///  <summary>
		///  Retrieves received data from a bulk endpoint.
		///  This routine is called automatically when myWinUsbDevice.ReceiveViaBulkTransfer
		///  returns. The routine calls several marshaling routines to access the main form.       
		///  </summary>
		///  
		///  <param name="ar"> An object containing status information about the 
		///  asynchronous operation.</param>
		///  
		private void GetBulkDataReceived(IAsyncResult ar)
		{
			UInt32 bytesRead = 0;
			var myEncoder = new System.Text.ASCIIEncoding();
			var success = false;

			Byte[] receivedDataBuffer = null;

			// Define a delegate using the IAsyncResult object.

			var deleg =
				((ReceiveFromDeviceDelegate)(ar.AsyncState));

			// Get the IAsyncResult object and the values of other paramaters that the
			// BeginInvoke method passed ByRef.

			deleg.EndInvoke
				(ref receivedDataBuffer,
				 ref bytesRead,
				 ref success, ar);

			// Display the received data in the form's list box.

			if ((ar.IsCompleted && success))
			{
				MyMarshalToForm("AddItemToListBox", "Data received via bulk transfer:");

				//  Convert the received bytes to a String for display.

				String receivedtext = myEncoder.GetString(receivedDataBuffer);

				MyMarshalToForm("AddItemToListBox", receivedtext);
			}
			else
			{
				MyMarshalToForm("AddItemToListBox", "The attempt to read bulk data has failed.");										
			}

			MyMarshalToForm("ScrollToBottomOfListBox", "");

			// Enable requesting another transfer.

			MyMarshalToForm("EnableCmdSendandReceiveViaBulkTransfers", "");
		}
		
		///  <summary>
		///  Initializes elements on the form.
		///  </summary>

		private void InitializeDisplay()
		{
			// Create dropdown list boxes.

			for (var i = 0; i <= 255; i++)
			{
				String byteValue = String.Format("{0:X2} ", i);

				FrmMy.CboInterruptOutByte0.Items.Insert(i, byteValue);
				FrmMy.CboInterruptOutByte1.Items.Insert(i, byteValue);
				FrmMy.CboControlWriteByte0.Items.Insert(i, byteValue);
				FrmMy.CboControlWriteByte1.Items.Insert(i, byteValue);
			}

			// Select a default value for each box

			FrmMy.CboInterruptOutByte0.SelectedIndex = 0;
			FrmMy.CboInterruptOutByte1.SelectedIndex = 128;
			FrmMy.CboControlWriteByte0.SelectedIndex = 65;
			FrmMy.CboControlWriteByte1.SelectedIndex = 53;

			if (!(_myWinUsbCommunications.IsWindowsXpOrLater()))
			{
				LstResults.Items.Add("The operating system is not Windows XP or later.");
				LstResults.Items.Add("The WinUsb driver requires Windows XP or later.");
			}
		}

		///  <summary>
		///  Enables accessing a form from another thread 
		///  </summary>
		///  
		///  <param name="action"> A String that names the action to perform on the form. </param>
		///  <param name="textToDisplay"> Text that the form displays or uses for 
		///  another purpose. Actions that don't use text ignore this parameter. </param>

		private void MyMarshalToForm(String action, String textToDisplay)
		{
			object[] args = { action, textToDisplay };

			//  The AccessForm routine contains the code that accesses the form.

			MarshalToForm marshalToFormDelegate = AccessForm;

			//  Execute AccessForm, passing the parameters in args.

			Invoke(marshalToFormDelegate, args);
		}

		///  <summary>
		///  Called when a WM_DEVICECHANGE message has arrived,
		///  indicating that a device has been attached or removed.
		///  </summary>
		///  
		///  <param name="m"> A message with information about the device. </param>

		internal void OnDeviceChange(Message m)
		{
			if ((m.WParam.ToInt32() == DeviceManagement.NativeMethods.DBT_DEVICEARRIVAL))
			{

				//  If WParam contains DBT_DEVICEARRIVAL, a device has been attached.
				//  Find out if it's the device we're communicating with.

				if (_myDeviceManagement.DeviceNameMatch(m, _myDevicePathName))
				{
					LstResults.Items.Add("My device attached.");
				}

			}
			else if ((m.WParam.ToInt32() ==
			          DeviceManagement.NativeMethods.DBT_DEVICEREMOVECOMPLETE))
			{
				//  If WParam contains DBT_DEVICEREMOVAL, a device has been removed.
				//  Find out if it's the device we're communicating with.

				if (_myDeviceManagement.DeviceNameMatch(m, _myDevicePathName))
				{

					LstResults.Items.Add("My device removed.");

					//  Set deviceReady False so on the next data-transfer attempt,
					//  FindMyDevice() will be called to look for the device 
					//  and get a new handle.

					_deviceReady = false;
				}
			}

			ScrollToBottomOfListBox();
		}

		///  <summary>
		///  Initiates a Control Read transfer. (Data stage is device to host.)
		///  </summary>
		///  
		private void RequestControlReadTransfer()
		{
			var dataStage = new Byte[2];

			// If the device hasn't been detected or was removed, look for the device.

			if (!(_deviceReady))
			{
				_deviceReady = FindMyDevice();
			}

			if (_deviceReady)
			{
				var success = _myWinUsbCommunications.DoControlReadTransfer(_winUsbHandle, ref dataStage);

				String formText;
				if (success)
				{
					formText = "Bytes received via Control Read transfer:";
					AccessForm("AddItemToListBox", formText);

					for (Int32 i = 0; i <= dataStage.Length - 1; i++)
					{
						formText = String.Format("{0:X2} ", dataStage[i]);
						AccessForm("AddItemToListBox", formText);
					}
				}
				else
				{
					formText = "Control Read transfer failed.";
					AccessForm("AddItemToListBox", formText);						
				}
			}
			ScrollToBottomOfListBox();
		}

		///  <summary>
		///  Initiates a Control Write transfer. (Data stage is host to device.)
		///  Converts hex strings from combo boxes to byte values for sending.
		///  </summary>
		///  
		private void RequestControlWriteTransfer()
		{
			var dataStage = new Byte[2];

			// If the device hasn't been detected, was removed, or timed out on a previous attempt
			// to access it, look for the device.

			if (!(_deviceReady))
			{
				_deviceReady = FindMyDevice();
			}

			if (_deviceReady)
			{
				// Get the value to send as a hex String.

				var comboBoxText = Convert.ToString(CboControlWriteByte0.SelectedItem).TrimEnd(null);

				// Convert the String to a byte.

				dataStage[0] = Convert.ToByte(String.Format("{0:X2}", comboBoxText), 16);

				// Get the value to send as a hex String.

				comboBoxText =
					Convert.ToString(CboControlWriteByte1.SelectedItem).TrimEnd(null);

				// Convert the String to a byte.

				dataStage[1] = Convert.ToByte(String.Format("{0:X2}", comboBoxText), 16);

				var success = _myWinUsbCommunications.DoControlWriteTransfer(_winUsbHandle, dataStage);

				String formText;
				if (success)
				{
					formText = "Control Write transfer completed successfully.";
				}
				else
				{
					formText = "Control Write transfer failed.";						
				}

				AccessForm("AddItemToListBox", formText);
			}
			ScrollToBottomOfListBox();
		}
		
		///  <summary>
		///  Initiates a read operation from a bulk IN endpoint.
		///  To enable reading without blocking the main thread, uses an asynchronous delegate.
		///  </summary>
		///  
		///  <param name="bytesToRead"> The number of bytes to read </param>		
		///  <param name="dataBuffer"> Buffer to hold the bytes read </param>
		///  <param name="bytesRead"> The number of bytes read </param>
		///  <param name="success"> True on success </param>		

		private void RequestToReceiveDataViaBulkTransfer(UInt32 bytesToRead, Byte[] dataBuffer, ref UInt32 bytesRead, ref Boolean success)
		{
			if (dataBuffer == null) throw new ArgumentNullException("dataBuffer");
			IAsyncResult ar = null;			

			//  Define a delegate for the ReadViaBulkTransfer method of WinUsbDevice.

			ReceiveFromDeviceDelegate myReceiveFromDeviceDelegate = _myWinUsbCommunications.ReceiveDataViaBulkTransfer;

			//  The BeginInvoke method calls MyWinUsbDevice.ReceiveViaBulkTransfer to attempt 
			//  to read data. The method has the same parameters as ReceiveViaBulkTransfer,
			//  plus two additional parameters:
			//  GetBulkDataReceived is the callback routine that executes when 
			//  ReceiveViaBulkTransfer returns.
			//  MyReceiveFromDeviceDelegate is the asynchronous delegate object.
			
			myReceiveFromDeviceDelegate.BeginInvoke
				(_winUsbHandle, 
				_myDeviceInfo, 					
				 bytesToRead,
				 ref dataBuffer,
				 ref bytesRead,
				 ref success,
				 GetBulkDataReceived,
				 myReceiveFromDeviceDelegate);
		}

		///  <summary>
		///  Initiates a read operation from an interrupt IN endpoint.
		///  Synchronous, blocks until data received or timeout.
		///  </summary>
		///  
		///  <param name="bytesToRead"> The number of bytes to read </param>		
		///  <param name="dataBuffer"> Buffer to hold the bytes read </param>
		///  <param name="bytesRead"> The number of bytes read </param>
		///  <param name="success"> True on success </param>  		

		private void RequestToReceiveDataViaInterruptTransfer(UInt32 bytesToRead, Byte[] dataBuffer, ref UInt32 bytesRead, ref Boolean success)
		{
			if (dataBuffer == null) throw new ArgumentNullException("dataBuffer");
			success = _myWinUsbCommunications.ReceiveDataViaInterruptTransfer
				(_winUsbHandle, 
				_myDeviceInfo,
				bytesToRead,
			    ref dataBuffer,
				ref bytesRead  
				);

			if (success)
			{
				MyMarshalToForm("AddItemToListBox", "Data received via interrupt transfer:");

				MyMarshalToForm("AddItemToListBox", " Received Data:");

				for (Int32 i = 0; i <= dataBuffer.GetUpperBound(0); i++)
				{
					// Convert the byte value to a 2-character hex String.

					var byteValue = String.Format("{0:X2} ", dataBuffer[i]);

					MyMarshalToForm("AddItemToListBox", " " + byteValue);
				}
			}
			else
			{
				MyMarshalToForm("AddItemToListBox", "The attempt to read interrupt data has failed.");					
			}

			MyMarshalToForm("ScrollToBottomOfListBox", "");

			// Enable requesting another transfer.

			MyMarshalToForm("EnableCmdSendandReceiveViaInterruptTransfers", "");
		}

		///  <summary>
		///  Initiates a write operation to a bulk OUT endpoint.
		///  To enable writing without blocking the main thread, uses an asynchronous delegate.
		///  </summary>
		///  
		///  <param name="bytesToSend"> The number of bytes to send </param> 
		///  <param name="dataBuffer"> Buffer to hold the bytes to send </param> 
		///  <param name="bytesSent"> The number of bytes send </param>
		///  <param name="success"> True on success </param>		

		private void RequestToSendDataViaBulkTransfer(UInt32 bytesToSend, Byte[] dataBuffer, ref UInt32 bytesSent, ref Boolean success)
		{
			if (dataBuffer == null) throw new ArgumentNullException("dataBuffer");
			IAsyncResult ar = null;

			//  Define a delegate for the ReadViaBulkTransfer method of WinUsbDevice.

			SendToDeviceDelegate mySendToDeviceDelegate = _myWinUsbCommunications.SendDataViaBulkTransfer;

			//  The BeginInvoke method calls MyWinUsbDevice.SendViaBulkTransfer to attempt 
			//  to read data. The method has the same parameters as SendViaBulkTransfer,
			//  plus two additional parameters:
			//  GetBulkDataSent is the callback routine that executes when 
			//  SendViaBulkTransfer returns.
			//  MySendToDeviceDelegate is the asynchronous delegate object.

			mySendToDeviceDelegate.BeginInvoke
				(_winUsbHandle, 
				_myDeviceInfo,					
				 bytesToSend,
				 dataBuffer,
				 ref bytesSent,
				 ref success,
				 GetBulkDataSent,
				 mySendToDeviceDelegate);
		}

		///  <summary>
		///  Initiates a write operation to an interrupt OUT endpoint.
		///  Synchronous, blocks until data sent or timeout.
		///  </summary>
		///  
		///  <param name="bytesToSend"> The number of bytes to send </param> 
		///  <param name="dataBuffer"> Buffer to hold the bytes to send </param> 
		///  <param name="bytesSent"> The number of bytes send </param>
		///  <param name="success"> True on success </param>	

		private void RequestToSendDataViaInterruptTransfer(UInt32 bytesToSend, Byte[] dataBuffer, ref UInt32 bytesSent, ref Boolean success)
		{
			if (dataBuffer == null) throw new ArgumentNullException("dataBuffer");
			success = _myWinUsbCommunications.SendDataViaInterruptTransfer
				(_winUsbHandle,
				_myDeviceInfo,						
				 bytesToSend,
				 dataBuffer,
				 ref bytesSent);

			if (success)
			{
				MyMarshalToForm("AddItemToListBox", "Data sent via interrupt transfer:");	
			}
			else
			{
				MyMarshalToForm("AddItemToListBox", "The attempt to send interrupt data has failed.");					
			}

			MyMarshalToForm("ScrollToBottomOfListBox", "");

			// Enable requesting another transfer.

			MyMarshalToForm("EnableCmdSendandReceiveViaInterruptTransfers", "");
		}

		///  <summary>
		///  Scroll to the bottom of the list box and trim if needed.
		///  </summary>

		private void ScrollToBottomOfListBox()
		{
			LstResults.SelectedIndex = LstResults.Items.Count - 1;

			// If the list box is getting too large, trim its contents.

			if (LstResults.Items.Count > 1000)
			{
				for (Int32 i = 1; i <= 800; i++)
				{
					LstResults.Items.RemoveAt(i);
				}
			}
		}

		///  <summary>
		///  Initiates sending data via a bulk transfer, then receiving data via a bulk transfer.
		///  </summary>

		private void SendAndReceiveViaBulkTransfers()
		{
			UInt32 bytesWritten = 0;
			var myEncoder = new System.Text.ASCIIEncoding();
			var success = false;				

			//  Get data to send from the textbox.

			var dataToSend = TxtBulkDataToSend.Text;

			if (dataToSend.Length > 64)
			{
				dataToSend = dataToSend.Substring(0, 64); 
			}

			//  Convert the String to a byte array.

			var databuffer = myEncoder.GetBytes(dataToSend);
				
			var bytesToSend = Convert.ToUInt32(databuffer.Length);

			// If the device hasn't been detected, look for it.

			if (!(_deviceReady))
			{
				_deviceReady = FindMyDevice();
			}

			var dataBuffer = new Byte[64];
			if (_deviceReady)
			{
				RequestToSendDataViaBulkTransfer(bytesToSend, databuffer, ref bytesWritten, ref success);

				UInt32 bytesRead = 0;
				UInt32 bytesToRead = Convert.ToUInt32(64);					

				RequestToReceiveDataViaBulkTransfer(bytesToRead, dataBuffer, ref bytesRead, ref success);					
			}
		}

		///  <summary>
		///  Initiates sending data via an interrupt transfer, then retrieving data 
		///  via an interrupt transfer.
		///  Converts hex strings from combo boxes to byte values for sending.
		///  </summary>

		private void SendAndReceiveViaInterruptTransfers()
		{
			UInt32 bytesWritten = 0;
			var dataBuffer = new Byte[2];
			var dataToSend = "";
			var formText = "";			
			Boolean success = false;

			//  Get bytes to send from combo boxes.

			// Get the value to send as a hex String.

			var comboBoxText = Convert.ToString(CboInterruptOutByte0.SelectedItem).TrimEnd(null);

			// Convert the String to a Byte.

			dataBuffer[0] = Convert.ToByte(String.Format("{0:X2}", comboBoxText), 16);

			// Get the value to send as a hex String.

			comboBoxText =
				Convert.ToString(CboInterruptOutByte1.SelectedItem).TrimEnd(null);

			// Convert the String to a Byte.

			dataBuffer[1] =
				Convert.ToByte(String.Format("{0:X2}", comboBoxText), 16);

			var bytesToSend = Convert.ToUInt32(dataBuffer.Length);

			var dataInBuffer = new Byte[2];
			UInt32 bytesRead = 0;
			var bytesToRead = Convert.ToUInt32(2);				


			// If the device hasn't been detected, look for it.

			if (!(_deviceReady))
			{
				_deviceReady = FindMyDevice();
			}

			if (_deviceReady)
			{
				RequestToSendDataViaInterruptTransfer(bytesToSend, dataBuffer, ref bytesWritten, ref success);				
				RequestToReceiveDataViaInterruptTransfer(bytesToRead, dataInBuffer, ref bytesRead, ref success);
			}
		}

		///  <summary>
		///  Perform actions that must execute when the program ends.
		///  </summary>

		private void Shutdown()
		{
			_myWinUsbCommunications.CloseDeviceHandle(_deviceHandle, _winUsbHandle);

			_myDeviceManagement.StopReceivingDeviceNotifications
				(_deviceNotificationHandle);
		}

		///  <summary>
		///  Perform actions that must execute when the program starts.
		///  </summary>

		private void Startup()
		{
			var myWinUsbCommunications = new WinUsbCommunications();
			InitializeDisplay();
		}

		///  <summary>
		///  Overrides WndProc to enable checking for and handling
		///  WM_DEVICECHANGE messages.
		///  </summary>
		///  
		///  <param name="m"> A Windows message.
		///  </param> 
		 
		protected override void WndProc(ref Message m)
		{
			// The OnDeviceChange routine processes WM_DEVICECHANGE messages.

			if (m.Msg == DeviceManagement.NativeMethods.WM_DEVICECHANGE)
			{
				OnDeviceChange(m);
			}
			// Let the base form process the message.

			base.WndProc(ref m);
		}
	}
}
